PROGRAM "osname"
VERSION "1.00"
'CONSOLE
'
' osname - Demo for XstGetOSName().
' Copyright (c) GPL 2017 Guy Lonne.
'
' ***** Description *****
' Display on several plateforms the string returned by XstGetOSName().
' - "Windows 10" on Windows 10;
' - "Windows 7"  on Windows 7;
' - "Windows XP" on Windows XP.
'
' ***** Versions *****
' 1.00-11jan17-GL-Generated GUI prototype using viXen v1.99u.
' Operating System: Windows 10
' Generation switches that are on:
' - Use Windows API
' - Use Windows XP Style
' - Use Debugging Functions
' - Use Text-only Push Buttons
' - Meticulous Cleanup
'
'
' ##############################
' #####  Import Libraries  #####
' ##############################
'
'
' WinAPI DLL headers
'
	IMPORT "kernel32"    ' Operating System
'
' ---Note: import gdi32 BEFORE shell32 and user32
	IMPORT "gdi32"       ' Graphic Device Interface
	IMPORT "shell32"     ' interface to Operating System
	IMPORT "user32"      ' Windows management
'
' ---Note: import comctl32 BEFORE comdlg32
	IMPORT "comctl32"    ' common controls; ==> initialize w/ InitCommonControlsEx ()
'	IMPORT "comdlg32"    ' standard dialog boxes (opening and saving files ...)
'
' XBLite headers
'
	IMPORT "xst"         ' Xblite Standard Library
	IMPORT "xsx"         ' Xblite Standard eXtended Library
'	IMPORT "xio"         ' console library
'
'
'
' ********************************************
' *****  INTERNAL FUNCTION DECLARATIONS  *****
' ********************************************
'
DECLARE FUNCTION Entry () ' program entry point

DECLARE FUNCTION CleanUp () ' program cleanup

DECLARE FUNCTION CreateWindows () ' create the windows of the program

DECLARE FUNCTION GuiTellApiError (msg$) ' display an API error message
DECLARE FUNCTION GuiTellRunError (msg$) ' display an XBLite error message

DECLARE FUNCTION InitGui () ' initialize win32 controls and libraries
DECLARE FUNCTION InitWindows () ' for initializations after CreateWindows()

DECLARE FUNCTION MessageLoop () ' main message loop

DECLARE FUNCTION NewChild (class$, text$, style, x, y, w, h, hParent, idCtr, exStyle) ' create a child window
DECLARE FUNCTION NewWindow (class$, titleBar$, style, x, y, w, h, exStyle) ' create a window

DECLARE FUNCTION RegisterWinClass (class$, addrProc, icon$, menu$) ' register the window class

DECLARE FUNCTION StartUp () ' program setup

DECLARE FUNCTION WapiKillFont (@hFont) ' release the logical font
DECLARE FUNCTION WapiSetFont (hCtr, hFont) ' apply font to control and redraw it
DECLARE FUNCTION WapiSetFontAndRedraw (hCtr, hFont, bRedraw) ' apply font to control, eventually redraw it

DECLARE FUNCTION WndProc (hWnd, wMsg, wParam, lParam) ' callback function for window #winMain
'
'
' *****************************************
' *****  SHARED CONSTANT DEFINITIONS  *****
' *****************************************
'
'
'
' ***** Constants used as control identificators for window #winMain *****
'
' control identificators for #winMain
$$lblOS = 101	' static 'OS Name:'
$$edtOS = 102	' edit control
'
'
' ***** Shared Program Constants *****
'
'
' ###################
' #####  Entry  #####
' ###################
'
' Program entry point.
'
FUNCTION Entry ()
	STATIC entry            ' ensure Entry is entered only one time

	IF entry THEN RETURN    ' enter once
	entry = $$TRUE          ' enter occured

'	XioCreateConsole ("", 50)		' create console
	bErr = StartUp ()		' initialize program and libraries
	IF bErr THEN XstAlert ("Entry: Can't initialize the program")

	bErr = InitGui ()		' initialize the GUI
	IF bErr THEN XstAlert ("Entry: Can't initialize the GUI")

	bErr = CreateWindows ()		' create the windows of the program
	IF bErr THEN XstAlert ("Entry: Can't create the windows of the program")

	InitWindows ()          ' for initializations after CreateWindows()
	MessageLoop ()          ' the message loop
	CleanUp ()              ' program cleanup

'	a$ = INLINE$ ("Press ENTER to exit >")
'	XioFreeConsole ()		' free console
	QUIT (0)

END FUNCTION
'
' Program cleanup.
'
FUNCTION CleanUp ()
	SHARED hInst
	SHARED hFontDefault

	' 1.Release logical font(s)

	WapiKillFont (@hFontDefault)

	' 2.Destroy window(s) and dialog(s)

	IF #winMain THEN
		SetLastError (0)
		ret = DestroyWindow (#winMain)		' destroy window #winMain
		IFZ ret THEN
			msg$ = "DestroyWindow: Can't destroy window #winMain"
			GuiTellApiError (msg$)
		ENDIF
	ENDIF
	#winMain = 0

	' 3.Unregister window class(es)

	SetLastError (0)
	ret = UnregisterClassA (&#winMain_class$, hInst)		' unregister window class #winMain_class$
	IFZ ret THEN
		msg$ = "UnregisterClassA: Can't unregister window class '" + #winMain_class$ + "'"
		GuiTellApiError (msg$)
	ENDIF

END FUNCTION
'
' ###########################
' #####  CreateWindows  #####
' ###########################
'
' Creates the windows of the program.
'
FUNCTION CreateWindows ()
	SHARED hInst

	' ***************** Begin window #winMain setup *****************
	' register the window class for window #winMain
	#winMain_class$ = PROGRAM$ (0) + "_winMain"
	addrProc = &WndProc ()
	SetLastError (0)
	bErr = RegisterWinClass (#winMain_class$, addrProc, "", "")		' register window class #winMain_class$
	IF bErr THEN
		msg$ = "RegisterWinClass: Can't register window class '" + #winMain_class$ + "'"
		bErr = GuiTellApiError (msg$)
		IF bErr THEN RETURN $$TRUE		' fail
	ENDIF

	' create window #winMain
	style = $$WS_OVERLAPPEDWINDOW
	#winMain = NewWindow (#winMain_class$, "XstGetOSName()", style, 0, 0, 314, 96, 0)		' create window #winMain
	IFZ #winMain THEN
		msg$ = "NewWindow: Can't create window #winMain"
		XstAlert (msg$)
		RETURN $$TRUE		' fail
	ENDIF

	' ***************** End window #winMain setup *****************

	XstCenterWindow (#winMain)		' auto-center the window
	ShowWindow (#winMain, $$SW_SHOWNORMAL)		' show the window

END FUNCTION
'
' Displays an API error message.
' RETURNs bErr: $$TRUE only if an error REALLY occurred.
'
' Usage:
'	SetLastError (0)
'	hImage = LoadImageA (0, &file$, $$IMAGE_BITMAP, 0, 0, $$LR_LOADFROMFILE)
'	IFZ hImage THEN
'		msg$ = "LoadImageA: Can't load image file " + file$
'		bErr = GuiTellApiError (msg$)
'		IF bErr THEN RETURN $$TRUE		' fail
'	ENDIF
'
FUNCTION GuiTellApiError (msg$)

	' get the last error code, then clear it
	errNum = GetLastError ()
	SetLastError (0)
	IFZ errNum THEN RETURN		' was OK!

	fmtMsg$ = "Last error code " + STRING$ (errNum) + ": "

	' set up FormatMessageA arguments
	dwFlags = $$FORMAT_MESSAGE_FROM_SYSTEM | $$FORMAT_MESSAGE_IGNORE_INSERTS
	lenBuf = 1020 : szBuf$ = NULL$ (lenBuf)
	ret = FormatMessageA (dwFlags, 0, errNum, 0, &szBuf$, lenBuf, 0)
	IFZ ret THEN
		fmtMsg$ = fmtMsg$ + "(unknown)"
	ELSE
		fmtMsg$ = fmtMsg$ + CSTRING$ (&szBuf$)
	ENDIF

	IFZ msg$ THEN msg$ = "Windows API error"
	fmtMsg$ = fmtMsg$ + $$CRLF$ + $$CRLF$ + msg$

	' get the OS name and version
	bErr = XstGetOSName (@osName$)
	IF bErr THEN
		st$ = "(unknown)"
	ELSE
		IFZ osName$ THEN osName$ = "(unknown)"
		st$ = osName$ + " ver."
		bErr = XstGetOSVersion (@major, @minor, @platformId, @version$, @platform$)
		IF bErr THEN
			st$ = st$ + " (unknown)"
		ELSE
			st$ = st$ + STR$ (major) + "." + STRING$ (minor) + "-" + platform$
		ENDIF
	ENDIF
	fmtMsg$ = fmtMsg$ + "\r\nOS: " + st$
	? fmtMsg$		' output message to an active console

	title$ = PROGRAM$ (0) + "-API Error"
	hwnd = GetActiveWindow ()
	MessageBoxA (hwnd, &fmtMsg$, &title$, $$MB_ICONSTOP)

	RETURN $$TRUE		' an error really occurred!

END FUNCTION
'
' Displays an XBLite error message.
' RETURNs $$TRUE only if an error really occurred.
'
FUNCTION GuiTellRunError (msg$)

	' get current error, then clear it
	errNum = ERROR (0)
	IFZ errNum THEN
		bErr = $$FALSE		' was OK!
	ELSE
		bErr = $$TRUE		' an error really occurred!
		'
		fmtMsg$ = "Error code " + STRING$ (errNum) + ", " + ERROR$ (errNum)
		'
		IFZ msg$ THEN msg$ = "Xblite library error"
		fmtMsg$ = fmtMsg$ + $$CRLF$ + $$CRLF$ + msg$
		? fmtMsg$		' output message to an active console
		'
		XstAlert (fmtMsg$)
	ENDIF

	RETURN bErr

END FUNCTION
'
' Initializes win32 controls and libraries.
' RETURNs bErr: $$TRUE on fail.
'
FUNCTION InitGui ()
	SHARED hInst
	SHARED hFontDefault		' default control font

	INITCOMMONCONTROLSEX icc

	SetLastError (0)
	IFZ hInst THEN
		hInst = GetModuleHandleA (0)		' a valid hInst ensures proper images loading
		IFZ hInst THEN
			msg$ = "InitGui: Can't get current instance handle"
			GuiTellApiError (msg$)
		ENDIF
	ENDIF

	' $$ICC_ANIMATE_CLASS      : animate
	' $$ICC_BAR_CLASSES        : toolbar, status bar, trackbar, tooltips
	' $$ICC_COOL_CLASSES       : rebar (coolbar) control
	' $$ICC_DATE_CLASSES       : month picker, date picker, time picker, up-down
	' $$ICC_HOTKEY_CLASS       : hotkey
	' $$ICC_INTERNET_CLASSES   : WIN32_IE >= 0x0400
	' $$ICC_LISTVIEW_CLASSES   : listview, header
	' $$ICC_PAGESCROLLER_CLASS : page scroller (WIN32_IE >= 0x0400)
	' $$ICC_PROGRESS_CLASS     : progress bar
	' $$ICC_TAB_CLASSES        : tab control, tooltips
	' $$ICC_TREEVIEW_CLASSES   : treeview, tooltips
	' $$ICC_UPDOWN_CLASS       : up-down
	' $$ICC_USEREX_CLASSES     : extended combo box
	' $$ICC_WIN95_CLASSES      : everything else

	' initialize the common control classes
	icc.dwSize = SIZE (icc)

	icc.dwICC = $$ICC_ANIMATE_CLASS      | _
              $$ICC_BAR_CLASSES        | _
              $$ICC_COOL_CLASSES       | _
              $$ICC_DATE_CLASSES       | _
              $$ICC_HOTKEY_CLASS       | _
              $$ICC_INTERNET_CLASSES   | _
              $$ICC_LISTVIEW_CLASSES   | _
              $$ICC_NATIVEFNTCTL_CLASS | _
              $$ICC_PAGESCROLLER_CLASS | _
              $$ICC_PROGRESS_CLASS     | _
              $$ICC_TAB_CLASSES        | _
              $$ICC_TREEVIEW_CLASSES   | _
              $$ICC_UPDOWN_CLASS       | _
              $$ICC_USEREX_CLASSES     | _
              $$ICC_WIN95_CLASSES

	InitCommonControlsEx (&icc)

	SetLastError (0)
	hFontDefault = GetStockObject ($$ANSI_VAR_FONT)		' get the ANSI default font
	IFZ hFontDefault THEN		' fail
		msg$ = "InitGui: Can't get the ANSI default font"
		GuiTellApiError (msg$)
	ENDIF

END FUNCTION
'
' Initializes after CreateWindows.
'
FUNCTION InitWindows ()

	UpdateWindow (#winMain)		' Update window #winMain

END FUNCTION
'
' The main message loop.
'
' Receives and handles messages sent to it from the system in reaction
' to User interactions and system events.
'
' RETURNs $$FALSE on receiving a QUIT message or $$TRUE on error.
'
FUNCTION MessageLoop ()
	MSG wMsg		' will be sent to window callback function when an event occurs

	' reserved to a stand-alone application
'	IF LIBRARY (0) THEN RETURN		' main program executes message loop

	' supervise system messages until
	' - the User decides to leave the application (RETURN $$FALSE)
	' - an error occurred (RETURNs bErr: $$TRUE on error)
	DO
		' retrieve next message from queue
		ret = GetMessageA (&wMsg, 0, 0, 0)
		SELECT CASE ret
			CASE 0 : RETURN		' QUIT message
			CASE -1 : RETURN $$TRUE		' fail
			CASE ELSE
				' deal with window messages
				hwnd = GetActiveWindow ()
				'
				IF (!IsWindow (hwnd)) || (!IsDialogMessageA (hwnd, &wMsg)) THEN
					' send only non-dialog messages
					' translate virtual-key messages into character messages
					' ex.: SHIFT + a is translated as "A"
					TranslateMessage (&wMsg)
					'
					' send message to window callback function
					DispatchMessageA (&wMsg)
				ENDIF
		END SELECT
	LOOP		' forever

END FUNCTION
'
' Creates a child window.
'
FUNCTION NewChild (class$, text$, style, x, y, w, h, hParent, idCtr, exStyle)
	SHARED hInst

	' create the child window
	newStyle = style | $$WS_CHILD | $$WS_VISIBLE

	SetLastError (0)
	hCtr = CreateWindowExA (exStyle, &class$, &text$, newStyle, x, y, w, h, hParent, idCtr, hInst, 0)
	IFZ hCtr THEN
		msg$ = "NewChild: Can't create the child window '" + text$ + "', control class '" + class$ + "'"
		GuiTellApiError (msg$)
	ENDIF

	RETURN hCtr

END FUNCTION
'
' Creates a window.
'
FUNCTION NewWindow (class$, titleBar$, style, x, y, w, h, exStyle)
	SHARED hInst

	SetLastError (0)
	hWin = CreateWindowExA (exStyle, &class$, &titleBar$, style, x, y, w, h, 0, 0, hInst, 0)
	IFZ hWin THEN
		msg$ = "NewWindow: Can't create the window, window class " + class$
		GuiTellApiError (msg$)
	ENDIF

	RETURN hWin

END FUNCTION
'
' Registers the window class.
' RETURNs bErr: $$TRUE on fail.
'
FUNCTION RegisterWinClass (class$, addrProc, icon$, menu$)
	SHARED hInst

	WNDCLASSEX wc		' window class
	OSVERSIONINFOEX os

	GetVersionExA (&os)
	IF os.dwMajorVersion < 6 THEN		' up to Windows XP
		wc.hbrBackground = $$COLOR_BTNFACE + 1
	ELSE		' Vista and above
		wc.hbrBackground = $$COLOR_WINDOW
	ENDIF

	wc.style = $$CS_HREDRAW | $$CS_VREDRAW | $$CS_OWNDC
	wc.lpfnWndProc = addrProc
	wc.cbSize = SIZE (WNDCLASSEX)
	wc.cbClsExtra = 0		' no extra bytes after the window class
	wc.cbWndExtra = 0		' structure or the window instance
	wc.hInstance = hInst

	IF icon$ THEN wc.hIcon = LoadIconA (hInst, &icon$)

	wc.hCursor = LoadCursorA (0, $$IDC_ARROW)
	wc.lpszMenuName = 0
	IF menu$ THEN wc.lpszMenuName = &menu$
	wc.lpszClassName = &class$

	SetLastError (0)
	ret = RegisterClassExA (&wc)
	IFZ ret THEN
		msg$ = "RegisterWinClass: Can't register the window class '" + class$ + "'"
		GuiTellApiError (msg$)
		RETURN $$TRUE		' fail
	ENDIF

END FUNCTION
'
' Program Setup.
'
' RETURNs $$TRUE on fail.
'
FUNCTION StartUp ()
	SHARED hInst

	SetLastError (0)
	hInst = GetModuleHandleA (0)		' get current instance handle
	IFZ hInst THEN
		msg$ = "GetModuleHandleA: Can't get current instance handle"
		bErr = GuiTellApiError (msg$)
		IF bErr THEN RETURN $$TRUE		' fail
	ENDIF

END FUNCTION
'
' Releases a logical font.
'
FUNCTION WapiKillFont (@r_hFont)

	SetLastError (0)

	IF r_hFont THEN
		DeleteObject (r_hFont)		' free memory space
	ELSE
		msg$ = "WapiKillFont: Ignore a NULL font handle"
		XstAlert (msg$)
		RETURN $$TRUE		' fail
	ENDIF

	r_hFont = 0

END FUNCTION
'
' Wrapper to WapiSetFontAndRedraw.
'
FUNCTION WapiSetFont (hCtr, hFont)

	bOK = WapiSetFontAndRedraw (hCtr, hFont, $$TRUE)		' redraw
	RETURN bOK

END FUNCTION
'
' Applies a font to control.
'
' hCtr    = control handle
' hFont   = logical font handle
' bRedraw = $$TRUE to redraw the control after setting the font
' RETURNs bOK: $$TRUE on success.
'
FUNCTION WapiSetFontAndRedraw (hCtr, hFont, bRedraw)

	SetLastError (0)
	bOK = $$FALSE

	SELECT CASE hCtr
		CASE 0
			msg$ = "WapiSetFontAndRedraw: Ignore a NULL control handle"
			XstAlert (msg$)
			'
		CASE ELSE
			' check hFont not null
			IFZ hFont THEN
				msg$ = "WapiSetFontAndRedraw: Ignore a NULL font handle"
				XstAlert (msg$)
				EXIT SELECT		' fail
			ENDIF
			'
			IF bRedraw THEN lParam = 1 ELSE lParam = 0
			SendMessageA (hCtr, $$WM_SETFONT, hFont, lParam)		' $$WM_SETFONT does not return a value
			bOK = $$TRUE
			'
	END SELECT

	RETURN bOK

END FUNCTION
'
' Callback function for window #winMain.
'
FUNCTION WndProc (hWnd, wMsg, wParam, lParam)
	SHARED hInst
	SHARED hFontDefault		' default control font

	RECT rect

	SELECT CASE wMsg
		CASE $$WM_CREATE		' event thrown by CreateWindowExA
			GOSUB winMain_Create
			RETURN
			'
		CASE $$WM_CLOSE		' closed by User
			DestroyWindow (hWnd)		' triggers $$WM_DESTROY
			RETURN
			'
		CASE $$WM_DESTROY		' being destroyed
			PostQuitMessage ($$WM_QUIT)		' quit program
			#winMain = 0
			RETURN
			'
		CASE $$WM_SIZE
			bErr = $$FALSE
			SetLastError (0)
			ret = GetWindowRect (hWnd, &rect)		' get rectangle of the window
			IFZ ret THEN
				msg$ = "GetWindowRect: Can't get rectangle of the window"
				bErr = GuiTellApiError (msg$)
				IF bErr THEN RETURN		' invalid handle
			ENDIF
			' ---
			winLeft = rect.left
			winTop = rect.top
			winWidth = rect.right - rect.left
			winHeight = rect.bottom - rect.top
			'
			bResize = $$FALSE
			' ( 314, 96 ) = minimum width and height of the window
			IF winWidth < 314 THEN
				' increase above the minimum width
				winWidth = 314
				bResize = $$TRUE
			ENDIF
			IF winHeight < 96 THEN
				' increase above the minimum height
				winHeight = 96
				bResize = $$TRUE
			ENDIF
			'
			IF bResize THEN SetWindowPos (hWnd, $$HWND_TOP, 0, 0, winWidth, winHeight, $$SWP_NOMOVE)
			'
			RETURN
			'
		CASE $$WM_COMMAND
			notifyCode = HIWORD (wParam)
			idCtr = LOWORD (wParam)
			hCtr = lParam
			'
			SELECT CASE idCtr
			END SELECT
			'
	END SELECT

	RETURN DefWindowProcA (hWnd, wMsg, wParam, lParam)		' message not handled

SUB winMain_Create

	#winMain = hWnd		' set the window global handle

	' *********** Begin Controls setup ***********

	' creating static $$lblOS, text 'OS Name:'
	label = NewChild ($$STATIC_CLASS, "OS Name:", 0, 10, 20, 56, 20, #winMain, $$lblOS, 0)		' create static $$lblOS
	IFZ label THEN
		msg$ = "NewChild: Can't create static $$lblOS"
		XstAlert (msg$)
	ENDIF

	WapiSetFont (label, hFontDefault)

	' creating edit control $$edtOS
	errNum = ERROR (0)
	bErr = XstGetOSName (@osName$)
	IF bErr THEN
		msg$ = "XstGetOSName: Can't get the OS name with XstGetOSName()"
		GuiTellRunError (msg$)
		osName$ = "(unknown)"
	ENDIF
	style = $$ES_READONLY | $$WS_TABSTOP | $$WS_BORDER
	#edtOS = NewChild ($$EDIT, osName$, style, 70, 18, 220, 20, #winMain, $$edtOS, $$WS_EX_CLIENTEDGE)		' create edit control $$edtOS
	IFZ #edtOS THEN
		msg$ = "NewChild: Can't create edit control $$edtOS"
		XstAlert (msg$)
	ENDIF

	WapiSetFont (#edtOS, hFontDefault)

	' *********** End Controls setup ***********

END SUB		' winMain_Create

END FUNCTION

END PROGRAM
